﻿//------------------------------------------------------------------------------
//  Copyright(C) 2022  Gerald Fahrnholz
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program. If not, see <http://www.gnu.org/licenses/>.
//
//    Contact: http://www.gerald-fahrnholz.eu/impressum.php
//------------------------------------------------------------------------------

using System.ComponentModel;
using System.Runtime.InteropServices;

namespace TestExecWin_VS2022
{
    [Guid("baad68e7-e126-4c26-912d-4440b6100f25")]
    public sealed class OptionPage : DialogPage
    {
        [DisplayName("Command line args")]
        [DefaultValue("")]
        [Category("General")]
        [Description("Start arguments passed to test executable.")]
        public string StartArgs { get; set; } = "";

        [DisplayName("Show more options within tool window")]
        [DefaultValue(true)]
        [Category("General")]
        [Description("Options are displayed at top of the tool window.")]
        public bool OptionsAreVisible { get; set; } = true;

        [DisplayName("Build solution before starting test")]
        [DefaultValue(false)]
        [Category("Advanced Options")]
        [Description("Ensure that latest code changes are compiled before test is started.")]
        public bool BuildSolutionBeforeStartingTest { get; set; } = false;

        [DisplayName("Limit CPU for StdOut")]
        [DefaultValue(false)]
        [Category("Advanced Options")]
        [Description("Keep Visual Studio GUI responsive while test executable produces bursts of output data (e.g. memory leak messages).")]
        public bool LimitCpuForStdOut { get; set; } = false;

        [DisplayName("Run until failure")]
        [DefaultValue(false)]
        [Category("Advanced Options")]
        [Description("Repeat execution of test until a failure occurs. Then you can check trace messages of sporadic error situation.")]
        public bool RunUntilFailure { get; set; } = false;

        [Browsable(false)]
        public bool LogIsVisible { get; set; } = true;

        [Browsable(false)]
        public int IdxGroupsAndFuncs { get; set; } = 0;

        [Browsable(false)]
        public int IdxSortOrder { get; set; } = 0;

        [Browsable(false)]
        public int IdxOutputLevel { get; set; } = 0;

        [Browsable(false)]
        public bool CheckForMemoryLeaks { get; set; } = false;

        [Browsable(false)]
        public int IdxMaxWaitTime { get; set; } = 0;

        [Browsable(false)]
        public int IdxCurrentTestApps { get; set; } = 0;

        [Browsable(false)]
        public string DescTestApps1 { get; set; } = "";

        [Browsable(false)]
        public string DescTestApps2 { get; set; } = "";

        [Browsable(false)]
        public string DescTestApps3 { get; set; } = "";

        [Browsable(false)]
        public string DescTestApps4 { get; set; } = "";

        [Browsable(false)]
        public string DescTestApps5 { get; set; } = "";

        [Browsable(false)]
        public string DescTestApps6 { get; set; } = "";

        [Browsable(false)]
        public string DescTestApps7 { get; set; } = "";

        [Browsable(false)]
        public string DescTestApps8 { get; set; } = "";

        [Browsable(false)]
        public string TestApps1 { get; set; } = "";

        [Browsable(false)]
        public string TestApps2 { get; set; } = "";

        [Browsable(false)]
        public string TestApps3 { get; set; } = "";

        [Browsable(false)]
        public string TestApps4 { get; set; } = "";

        [Browsable(false)]
        public string TestApps5 { get; set; } = "";

        [Browsable(false)]
        public string TestApps6 { get; set; } = "";

        [Browsable(false)]
        public string TestApps7 { get; set; } = "";

        [Browsable(false)]
        public string TestApps8 { get; set; } = "";


        // Notify MyToolWindow only if settings are changed from elsewhere
        private bool saveCalledFromMyToolWindow = false;

        public void SaveSettingsFromToolWindow()
        {
            saveCalledFromMyToolWindow = true;
            base.SaveSettingsToStorage();
            saveCalledFromMyToolWindow = false;

            // Settings are changed from within ToolWindow. No further notification needed.
        }
        public override void SaveSettingsToStorage()
        {
            // Remark: this method may be called by framework
            // when SaveSettingsFromToolWindow is called

            base.SaveSettingsToStorage();

            // Settings are changed from VS options menu, inform ToolWindow
            if ((MyGlobals.myMainEvents != null) && !saveCalledFromMyToolWindow)
            {
                MyGlobals.myMainEvents.OnSettingsHaveChanged();
            }
        }

        public string GetDescTestAppsForIdx(int idx)
        {
            string retVal = "";

            switch (idx)
            {
                case 0:
                    retVal = DescTestApps1;
                    break;
                case 1:
                    retVal = DescTestApps2;
                    break;
                case 2:
                    retVal = DescTestApps3;
                    break;
                case 3:
                    retVal = DescTestApps4;
                    break;
                case 4:
                    retVal = DescTestApps5;
                    break;
                case 5:
                    retVal = DescTestApps6;
                    break;
                case 6:
                    retVal = DescTestApps7;
                    break;
                default:
                    retVal = DescTestApps8;
                    break;
            }
            if (retVal == null || retVal.Length == 0)
            {
                retVal = $"{idx + 1}. Set of test apps (rename this)";
            }
            return retVal;
        }
        public string GetTestAppsForIdx(int idx)
        {
            string retVal = "";

            switch (idx)
            {
                case 0:
                    retVal = TestApps1;
                    break;
                case 1:
                    retVal = TestApps2;
                    break;
                case 2:
                    retVal = TestApps3;
                    break;
                case 3:
                    retVal = TestApps4;
                    break;
                case 4:
                    retVal = TestApps5;
                    break;
                case 5:
                    retVal = TestApps6;
                    break;
                case 6:
                    retVal = TestApps7;
                    break;
                default:
                    retVal = TestApps8;
                    break;
            }

            if (retVal == null || retVal.Length == 0)
            {
                retVal = $@"// App set {idx + 1} of 8. Enter each test app on a single line
// Empty lines and lines starting with '//' are ignored
C:\YourTargetDir\Debug\YourTestAppOne.exe - optionalArgs
C:\YourTargetDir\Debug\YourNextTestAppTwo.exe
// To automatically generate entries here check import
// possibilities via button 'Export/Import'";
            }
            return retVal;
        }
        public void SaveTestAppsForIdx(int idx, string description, string testApps)
        {
            switch (idx)
            {
                case 0:
                    DescTestApps1 = description;
                    TestApps1 = testApps;
                    break;
                case 1:
                    DescTestApps2 = description;
                    TestApps2 = testApps;
                    break;
                case 2:
                    DescTestApps3 = description;
                    TestApps3 = testApps;
                    break;
                case 3:
                    DescTestApps4 = description;
                    TestApps4 = testApps;
                    break;
                case 4:
                    DescTestApps5 = description;
                    TestApps5 = testApps;
                    break;
                case 5:
                    DescTestApps6 = description;
                    TestApps6 = testApps;
                    break;
                case 6:
                    DescTestApps7 = description;
                    TestApps7 = testApps;
                    break;
                default:
                    DescTestApps8 = description;
                    TestApps8 = testApps;
                    break;
            }
            saveCalledFromMyToolWindow = true;
            base.SaveSettingsToStorage();
            saveCalledFromMyToolWindow = false;
        }
    }
}