﻿//------------------------------------------------------------------------------
//  Copyright(C) 2022  Gerald Fahrnholz
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program. If not, see <http://www.gnu.org/licenses/>.
//
//    Contact: http://www.gerald-fahrnholz.eu/impressum.php
//------------------------------------------------------------------------------

using System.Runtime.InteropServices;

namespace TestExecWin_VS2022
{
    public enum AppType { NO_TEST_FOUND, TTB, BOOST };

    public class TestGroupEntry
    {
        public bool isBoostGroup { get; set; }
        public string fileFullPath { get; set; }
        public int lineNum { get; set; }
        public System.Collections.Generic.List<string> testGroups; // fulll hierarchy of test groups

        public TestGroupEntry(bool in_isBoostGroup)
        {
            isBoostGroup = in_isBoostGroup;
            testGroups = new System.Collections.Generic.List<string>();
        }
        public void AddTestGroupHierarchy(System.Collections.Generic.List<string> in_testGroupHierarchy)
        {
            foreach (string grp in in_testGroupHierarchy)
            {
                testGroups.Add(grp);
            }
        }
        public string GetTestGroupHierarchyString()
        {
            string allTestGroups = "";
            foreach (string grp in testGroups)
            {
                if (allTestGroups != "")
                {
                    allTestGroups += "/";
                }
                allTestGroups += grp;
            }
            return allTestGroups;
        }
        public string GetDisplayString()
        {
            string prefix = "";
            for (int i = 1; i < testGroups.Count; ++i)
            {
                prefix += "    ";
            }
            if (testGroups.Count == 0)
                return prefix + "<default>";
            else
                return prefix + testGroups[testGroups.Count - 1];
        }
        public string GetCmdString()
        {
            string cmdString = GetTestGroupHierarchyString();
            if (cmdString != "")
            {
                cmdString = (isBoostGroup ? "--run_test=" : "-selectTestFile ") + cmdString;
            }
            return cmdString;
        }
    }

    public class TestFuncEntry
    {
        public bool isBoostFunc { get; set; }
        public string testFunc { get; set; }
        public string fileFullPath { get; set; }
        public int lineNum { get; set; }
        public System.Collections.Generic.List<string> testGroups; // hierarchy of test groups

        public TestFuncEntry(bool in_isBoostFunc)
        {
            isBoostFunc = in_isBoostFunc;
            testFunc = "";
            fileFullPath = "";
            lineNum = 0;
            testGroups = new System.Collections.Generic.List<string>(); // hierarchy of test groups
        }
        public void AddTestGroupHierarchy(System.Collections.Generic.List<string> in_testGroupHierarchy)
        {
            if (in_testGroupHierarchy.Count == 0)
            {
                testGroups.Add(ProjectInfo.NO_TESTSUITE_STR);
            }
            else
            {
                foreach (string grp in in_testGroupHierarchy)
                {
                    testGroups.Add(grp);
                }
            }
        }
        public string GetTestGroupHierarchyString()
        {
            string allTestGroups = "";
            foreach (string grp in testGroups)
            {
                if (allTestGroups != "")
                {
                    allTestGroups += "/";
                }
                allTestGroups += grp;
            }
            return allTestGroups;
        }
        public string GetDisplayString()
        {
            return testFunc;
        }
        public string GetCmdString()
        {
            if (isBoostFunc)
            {
                string cmdString = GetTestGroupHierarchyString();
                if (cmdString == ProjectInfo.NO_TESTSUITE_STR)
                {
                    cmdString="";
                }
                if (cmdString != "")
                {
                    cmdString += "/";
                }
                cmdString += testFunc;
                return "--run_test=" + cmdString;
            }
            else // TTB
            {
                return "-selectTestFunc " + testFunc;
            }
        }
    }

    public class ProjectInfo
    {
        public string solutionFullPath { get; set; }
        public string project { get; set; }
        public string config { get; set; }
        public string platform { get; set; }
        public string sourceDirPath { get; set; }
        public string targetDirPath { get; set; }
        public string fullExePath { get; set; }
        public AppType appType { get; set; }
        public System.Collections.Generic.List<TestGroupEntry> testGroups = new System.Collections.Generic.List<TestGroupEntry>();
        public System.Collections.Generic.List<TestFuncEntry> testFuncs = new System.Collections.Generic.List<TestFuncEntry>();
        public bool testFuncWithoutTestGroupFound { get; set; }
        public const string NO_TESTSUITE_STR = "[no-test-suite]";
        public ProjectInfo()
        {
            solutionFullPath = "<not set>";
            project = "<not set>";
            config = "<not set>";
            platform = "<not set>";
            sourceDirPath = "<not set>";
            targetDirPath = "<not set>";
            fullExePath = "<not set>";
            appType = AppType.NO_TEST_FOUND;
            testFuncWithoutTestGroupFound = false;
        }

        public string GetExePath()
        {
            return fullExePath;
        }
    }
}